import os

from pydantic import SecretStr
from pytest_mock import MockerFixture

from src.pyamldantic import YAMLConfig

from .fixtures import Schema


def test_load(mocker: MockerFixture) -> None:
    mocker.patch.dict(
        os.environ,
        {
            "DATABASE_HOST": "foo",
            "DATABASE_NAME": "bar",
            "DATABASE_PASSWORD": "baz",
            "DATABASE_PORT": "5432",
            "DATABASE_USER": "qux",
        },
    )

    config = YAMLConfig.load("tests/fixtures/config.yaml", schema=Schema)
    assert config.database.host == os.environ["DATABASE_HOST"]
    assert config.database.name == os.environ["DATABASE_NAME"]
    assert config.database.password == SecretStr(os.environ["DATABASE_PASSWORD"])
    assert config.database.port == int(os.environ["DATABASE_PORT"])
    assert config.database.user == os.environ["DATABASE_USER"]
    assert config.database.ssl is False
    assert config.database.timeout is None
    assert config.environment == "testing"
    assert config.is_debug



from pathlib import Path

import pytest
from pytest_mock import MockerFixture

from src.pyamldantic import YAMLConfig
from src.pyamldantic.exceptions import InvalidYAMLError

from .fixtures import Schema


def test_invalid_yaml(mocker: MockerFixture) -> None:
    mocker.patch.object(Path, "open", mocker.mock_open(read_data="key: value:"))

    with pytest.raises(InvalidYAMLError):
        YAMLConfig.load("tests/fixtures/config.yaml", schema=Schema)



import pytest

from src.pyamldantic import YAMLConfig

from .fixtures import Schema


def test_file_not_found() -> None:
    with pytest.raises(FileNotFoundError):
        YAMLConfig.load("tests/fixtures/foo.yaml", schema=Schema)



from pydantic import BaseModel, SecretStr


class DatabaseSchema(BaseModel):  # noqa: D101
    host: str
    name: str
    password: SecretStr
    port: int
    user: str
    ssl: bool = False
    timeout: int | None = None


class Schema(BaseModel):  # noqa: D101
    database: DatabaseSchema
    environment: str
    is_debug: bool



import pytest

from src.pyamldantic import YAMLConfig
from src.pyamldantic.exceptions import MissingEnvironmentVariableError

from .fixtures import Schema


def test_missing_environment_variable() -> None:
    with pytest.raises(MissingEnvironmentVariableError):
        YAMLConfig.load("tests/fixtures/config.yaml", schema=Schema)



class InvalidYAMLError(Exception):
    """Raised when the config file is not a valid YAML file."""

    def __init__(self, path: str, e: Exception) -> None:  # noqa: D107
        super().__init__(f"{path} is not a valid YAML file: {e}")


class MissingEnvironmentVariableError(Exception):
    """Raised when an expected environment variable is missing."""

    def __init__(self, var: str) -> None:  # noqa: D107
        super().__init__(f"missing expected environment variable: `{var}`")


class SchemaValidationError(Exception):
    """Raised when the config file does not match the expected schema."""

    def __init__(self, path: str, e: Exception) -> None:  # noqa: D107
        super().__init__(f"{path} does not match the expected schema: {e}")
